@@ -4,7 +4,7 @@
  * configuration of Horde applications, writing conf.php files from
  * conf.xml source files, generating user interfaces, etc.
  *
- * Copyright 2002-2011 Horde LLC (http://www.horde.org/)
+ * Copyright 2002-2013 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
@@ -138,7 +138,6 @@ class Horde_Config
     }
 
     /**
-     * @since Horde_Core 1.1.0
      */
     public function configFile()
     {
@@ -202,12 +201,13 @@ class Horde_Config
 
         /* Parse additional config files. */
         foreach (glob($path . '/conf.d/*.xml') as $additional) {
+            $dom = new DOMDocument();
             $dom->load($additional);
             $root = $dom->documentElement;
             if ($root->hasChildNodes()) {
                 $tree = array();
                 $this->_parseLevel($tree, $root->childNodes, '');
-                $this->_xmlConfigTree = Horde_Array::replaceRecursive($this->_xmlConfigTree, $tree);
+                $this->_xmlConfigTree = array_replace_recursive($this->_xmlConfigTree, $tree);
             }
         }
 
@@ -271,8 +271,6 @@ class Horde_Config
      * Generates and writes the content of the application's configuration
      * file.
      *
-     * @since Horde_Core 1.1.0
-     *
      * @param Horde_Variables $formvars  The processed configuration form
      *                                   data.
      * @param string $php                The content of the generated
@@ -288,9 +286,9 @@ class Horde_Config
         $configFile = $this->configFile();
         if (file_exists($configFile)) {
             if (@copy($configFile, $path . '/conf.bak.php')) {
-                $GLOBALS['notification']->push(sprintf(_("Successfully saved the backup configuration file %s."), Horde_Util::realPath($path . '/conf.bak.php')), 'horde.success');
+                $GLOBALS['notification']->push(sprintf(Horde_Core_Translation::t("Successfully saved the backup configuration file %s."), Horde_Util::realPath($path . '/conf.bak.php')), 'horde.success');
             } else {
-                $GLOBALS['notification']->push(sprintf(_("Could not save the backup configuration file %s."), Horde_Util::realPath($path . '/conf.bak.php')), 'horde.warning');
+                $GLOBALS['notification']->push(sprintf(Horde_Core_Translation::t("Could not save the backup configuration file %s."), Horde_Util::realPath($path . '/conf.bak.php')), 'horde.warning');
             }
         }
         if ($fp = @fopen($configFile, 'w')) {
@@ -298,6 +296,7 @@ class Horde_Config
             fwrite($fp, $php);
             fclose($fp);
             $GLOBALS['registry']->rebuild();
+            $GLOBALS['notification']->push(sprintf(Horde_Core_Translation::t("Successfully wrote %s"), Horde_Util::realPath($configFile)), 'horde.success');
             return true;
         }
 
@@ -356,6 +355,10 @@ class Horde_Config
         }
 
         foreach ($section as $name => $configitem) {
+            if (is_array($configitem) && isset($configitem['tab'])) {
+                continue;
+            }
+
             $prefixedname = empty($prefix)
                 ? $name
                 : $prefix . '|' . $name;
@@ -741,26 +744,39 @@ class Horde_Config
     protected function _configLDAP($ctx, $node = null,
                                    $switchname = 'driverconfig')
     {
+        if ($node) {
+            $xpath = new DOMXPath($node->ownerDocument);
+        }
+
         $fields = array(
             'hostspec' => array(
                 '_type' => 'text',
                 'required' => true,
                 'desc' => 'LDAP server/hostname',
-                'default' => $this->_default($ctx . '|hostspec', '')
+                'default' => $this->_default(
+                    $ctx . '|hostspec',
+                    $node ? ($xpath->evaluate('string(configstring[@name="hostspec"])', $node) ?: '') : ''
+                )
             ),
 
             'port' => array(
                 '_type' => 'int',
                 'required' => false,
                 'desc' => 'Port on which LDAP is listening, if non-standard',
-                'default' => $this->_default($ctx . '|port', null)
+                'default' => $this->_default(
+                    $ctx . '|port',
+                    $node ? ($xpath->evaluate('string(configinteger[@name="port"])', $node) ?: null) : null
+                )
             ),
 
             'tls' => array(
                 '_type' => 'boolean',
                 'required' => false,
                 'desc' => 'Use TLS to connect to the server?',
-                'default' => $this->_default($ctx . '|tls', false)
+                'default' => $this->_default(
+                    $ctx . '|tls',
+                    $node ? ($xpath->evaluate('string(configboolean[@name="tls"])', $node) ?: false) : false
+                )
             ),
 
             'version' => array(
@@ -768,6 +784,10 @@ class Horde_Config
                 'required' => true,
                 'quote' => false,
                 'desc' => 'LDAP protocol version',
+                'default' => $this->_default(
+                    $ctx . '|version',
+                    $node ? ($xpath->evaluate('normalize-space(configswitch[@name="version"]/text())', $node) ?: 3) : 3
+                ),
                 'switch' => array(
                     '2' => array(
                         'desc' => '2 (deprecated)',
@@ -778,12 +798,14 @@ class Horde_Config
                         'fields' => array()
                     )
                 ),
-                'default' => $this->_default($ctx . '|version', 3)
             ),
 
             'bindas' => array(
                 'desc' => 'Bind to LDAP as which user?',
-                'default' => $this->_default($ctx . '|bindas', 'admin'),
+                'default' => $this->_default(
+                    $ctx . '|bindas',
+                    $node ? ($xpath->evaluate('normalize-space(configswitch[@name="bindas"]/text())', $node) ?: 'admin') : 'admin'
+                ),
                 'switch' => array(
                     'anon' => array(
                         'desc' => 'Bind anonymously',
@@ -797,24 +819,35 @@ class Horde_Config
                                     '_type' => 'text',
                                     'required' => false,
                                     'desc' => 'DN used to bind for searching the user\'s DN (leave empty for anonymous bind)',
-                                    'default' => $this->_default($ctx . '|binddn', '')
+                                    'default' => $this->_default(
+                                        $ctx . '|user|binddn',
+                                        $node ? ($xpath->evaluate('string(configsection/configstring[@name="binddn"])', $node) ?: '') : ''
+                                    )
                                 ),
                                 'bindpw' => array(
                                     '_type' => 'text',
                                     'required' => false,
                                     'desc' => 'Password for bind DN',
-                                    'default' => $this->_default($ctx . '|bindpw', '')
+                                    'default' => $this->_default(
+                                        $ctx . '|user|bindpw',
+                                        $node ? ($xpath->evaluate('string(configsection/configstring[@name="bindpw"])', $node) ?: '') : ''
+                                    )
                                 ),
                                 'uid' => array(
                                     '_type' => 'text',
                                     'required' => true,
                                     'desc' => 'The username search key (set to samaccountname for AD).',
-                                    'default' => $this->_default($ctx . '|user|uid', 'uid')
+                                    'default' => $this->_default(
+                                        $ctx . '|user|uid',
+                                        $node ? ($xpath->evaluate('string(configsection/configstring[@name="uid"])', $node) ?: 'uid') : 'uid'
+                                    )
                                 ),
                                 'filter_type' => array(
                                     'required' => false,
                                     'desc' => 'How to specify a filter for the user lists.',
-                                    'default' => $this->_default($ctx . '|user|filter_type', 'objectclass'),
+                                    'default' => $this->_default(
+                                        $ctx . '|user|filter_type',
+                                        $node ? ($xpath->evaluate('normalize-space(configsection/configswitch[@name="filter_type"]/text())', $node) ?: 'objectclass') : 'objectclass'),
                                     'switch' => array(
                                         'filter' => array(
                                             'desc' => 'LDAP filter string',
@@ -823,7 +856,10 @@ class Horde_Config
                                                     '_type' => 'text',
                                                     'required' => true,
                                                     'desc' => 'The LDAP filter string used to search for users.',
-                                                    'default' => $this->_default($ctx . '|user|filter', '(objectClass=*)')
+                                                    'default' => $this->_default(
+                                                        $ctx . '|user|filter',
+                                                        $node ? ($xpath->evaluate('string(configsection/configstring[@name="filter"])', $node) ?: '(objectClass=*)') : '(objectClass=*)'
+                                                    )
                                                 ),
                                             ),
                                         ),
@@ -834,7 +870,9 @@ class Horde_Config
                                                     '_type' => 'stringlist',
                                                     'required' => true,
                                                     'desc' => 'The objectclass filter used to search for users. Can be a single objectclass or a comma-separated list.',
-                                                    'default' => implode(', ', $this->_default($ctx . '|user|objectclass', array('*')))
+                                                    'default' => implode(', ', $this->_default(
+                                                        $ctx . '|user|objectclass',
+                                                        $node ? ($xpath->evaluate('string(configsection/configlist[@name="objectclass"])', $node) ?: array('*')) : array('*')))
                                                 ),
                                             ),
                                         ),
@@ -850,13 +888,18 @@ class Horde_Config
                                 '_type' => 'text',
                                 'required' => true,
                                 'desc' => 'DN used to bind to LDAP',
-                                'default' => $this->_default($ctx . '|binddn', '')
+                                'default' => $this->_default(
+                                    $ctx . '|binddn',
+                                    $node ? ($xpath->evaluate('string(configsection/configstring[@name="binddn"])', $node) ?: '') : ''
+                                )
                             ),
                             'bindpw' => array(
                                 '_type' => 'text',
                                 'required' => true,
                                 'desc' => 'Password for bind DN',
-                                'default' => $this->_default($ctx . '|bindpw', '')
+                                'default' => $this->_default(
+                                    $ctx . '|bindpw',
+                                    $node ? ($xpath->evaluate('string(configsection/configstring[@name="bindpw"])', $node) ?: '') : '')
                             )
                         )
                     ),
@@ -874,7 +917,10 @@ class Horde_Config
         if (isset($node) && $node->getAttribute('baseconfig') == 'true') {
             return array(
                 'desc' => 'Use LDAP?',
-                'default' => $this->_default($ctx . '|' . $node->getAttribute('switchname'), false),
+                'default' => $this->_default(
+                    $ctx . '|' . $node->getAttribute('switchname'),
+                    $node ? ($xpath->evaluate('normalize-space(text())', $node) ?: false) : false
+                ),
                 'switch' => array(
                     'false' => array(
                         'desc' => 'No',
@@ -893,13 +939,18 @@ class Horde_Config
                 '_type' => 'text',
                 'required' => true,
                 'desc' => 'Base DN',
-                'default' => $this->_default($ctx . '|basedn', '')
+                'default' => $this->_default(
+                    $ctx . '|basedn',
+                    $node ? ($xpath->evaluate('string(configstring[@name="basedn"])', $node) ?: '') : ''
+                )
             ),
             'scope' => array(
                 '_type' => 'enum',
                 'required' => true,
                 'desc' => 'Search scope',
-                'default' => $this->_default($ctx . '|scope', ''),
+                'default' => $this->_default(
+                    $ctx . '|scope',
+                    $node ? ($xpath->evaluate('normalize-space(configenum[@name="scope"]/text())', $node) ?: '') : ''),
                 'values' => array(
                     'sub' => 'Subtree search',
                     'one' => 'One level'),
@@ -949,58 +1000,84 @@ class Horde_Config
      */
     public function configSQL($ctx, $node = null, $switchname = 'driverconfig')
     {
+        if ($node) {
+            $xpath = new DOMXPath($node->ownerDocument);
+        }
+
         $persistent = array(
             '_type' => 'boolean',
             'required' => false,
             'desc' => 'Request persistent connections?',
-            'default' => $this->_default($ctx . '|persistent', false)
+            'default' => $this->_default(
+                $ctx . '|persistent',
+                $node ? ($xpath->evaluate('string(configboolean[@name="persistent"])', $node) ?: false) : false
+            )
         );
 
         $hostspec = array(
             '_type' => 'text',
             'required' => true,
             'desc' => 'Database server/host',
-            'default' => $this->_default($ctx . '|hostspec', '')
+            'default' => $this->_default(
+                $ctx . '|hostspec',
+                $node ? ($xpath->evaluate('string(configstring[@name="hostspec"])', $node) ?: '') : ''
+            )
         );
 
         $username = array(
             '_type' => 'text',
             'required' => true,
             'desc' => 'Username to connect to the database as',
-            'default' => $this->_default($ctx . '|username', '')
+            'default' => $this->_default(
+                $ctx . '|username',
+                $node ? ($xpath->evaluate('string(configstring[@name="username"])', $node) ?: '') : ''
+            )
         );
 
         $password = array(
             '_type' => 'text',
             'required' => false,
             'desc' => 'Password to connect with',
-            'default' => $this->_default($ctx . '|password', '')
+            'default' => $this->_default(
+                $ctx . '|password',
+                $node ? ($xpath->evaluate('string(configstring[@name="password"])', $node) ?: '') :  ''
+            )
         );
 
         $database = array(
             '_type' => 'text',
             'required' => true,
             'desc' => 'Database name to use',
-            'default' => $this->_default($ctx . '|database', '')
+            'default' => $this->_default(
+                $ctx . '|database',
+                $node ? ($xpath->evaluate('string(configstring[@name="database"])', $node) ?: '') : ''
+            )
         );
 
         $socket = array(
             '_type' => 'text',
             'required' => false,
             'desc' => 'Location of UNIX socket',
-            'default' => $this->_default($ctx . '|socket', '')
+            'default' => $this->_default(
+                $ctx . '|socket',
+                $node ? ($xpath->evaluate('string(configstring[@name="socket"])', $node) ?: '') : ''
+            )
         );
 
         $port = array(
             '_type' => 'int',
             'required' => false,
             'desc' => 'Port the DB is running on, if non-standard',
-            'default' => $this->_default($ctx . '|port', null)
+            'default' => $this->_default(
+                $ctx . '|port',
+                $node ? ($xpath->evaluate('string(configinteger[@name="port"])', $node) ?: null) : null)
         );
 
         $protocol = array(
             'desc' => 'How should we connect to the database?',
-            'default' => $this->_default($ctx . '|protocol', 'unix'),
+            'default' => $this->_default(
+                $ctx . '|protocol',
+                $node ? ($xpath->evaluate('normalize-space(configswitch[@name="protocol"]/text())', $node) ?: 'unix') : 'unix'),
             'switch' => array(
                 'unix' => array(
                     'desc' => 'UNIX Sockets',
@@ -1019,48 +1096,47 @@ class Horde_Config
         );
 
         $mysql_protocol = $protocol;
-        $mysql_protocol['switch']['tcp']['fields']['port']['default'] = $this->_default($ctx . '|port', 3306);
+        $mysql_protocol['switch']['tcp']['fields']['port']['default'] =
+            $this->_default(
+                $ctx . '|port',
+                $node ? ($xpath->evaluate('string(configinteger[@name="port"])', $node) ?: 3306) : 3306
+            );
 
         $charset = array(
             '_type' => 'text',
             'required' => true,
             'desc' => 'Internally used charset',
-            'default' => $this->_default($ctx . '|charset', 'utf-8')
+            'default' => $this->_default(
+                $ctx . '|charset',
+                $node ? ($xpath->evaluate('string(configstring[@name="charset"])', $node) ?: 'utf-8') : 'utf-8')
         );
 
         $ssl = array(
             '_type' => 'boolean',
             'required' => false,
             'desc' => 'Use SSL to connect to the server?',
-            'default' => $this->_default($ctx . '|ssl', false)
+            'default' => $this->_default(
+                $ctx . '|ssl',
+                $node ? ($xpath->evaluate('string(configboolean[@name="ssl"])', $node) ?: false) : false)
         );
 
         $ca = array(
             '_type' => 'text',
             'required' => false,
             'desc' => 'Certification Authority to use for SSL connections',
-            'default' => $this->_default($ctx . '|ca', '')
-        );
-
-        $read_hostspec = array(
-            '_type' => 'text',
-            'required' => true,
-            'desc' => 'Read database server/host',
-            'default' => $this->_default($ctx . '|read|hostspec', '')
-        );
-
-        $read_port = array(
-            '_type' => 'int',
-            'required' => false,
-            'desc' => 'Port the read DB is running on, if non-standard',
-            'default' => $this->_default($ctx . '|read|port', null)
+            'default' => $this->_default(
+                $ctx . '|ca',
+                $node ? ($xpath->evaluate('string(configstring[@name="ca"])', $node) ?: '') : ''
+            )
         );
 
         $splitread = array(
             '_type' => 'boolean',
             'required' => false,
             'desc' => 'Split reads to a different server?',
-            'default' => $this->_default($ctx . '|splitread', 'false'),
+            'default' => $this->_default(
+                $ctx . '|splitread',
+                $node ? ($xpath->evaluate('normalize-space(configswitch[@name="splitread"]/text())', $node) ?: 'false') : 'false'),
             'switch' => array(
                 'false' => array(
                     'desc' => 'Disabled',
@@ -1085,7 +1161,10 @@ class Horde_Config
         $custom_fields = array(
             'required' => true,
             'desc' => 'What database backend should we use?',
-            'default' => $this->_default($ctx . '|phptype', ''),
+            'default' => $this->_default(
+                $ctx . '|phptype',
+                $node ? $node->getAttribute('default') : ''
+            ),
             'switch' => array(
                 'false' => array(
                     'desc' => '[None]',
@@ -1102,7 +1181,7 @@ class Horde_Config
                         'charset' => $charset,
                         'ssl' => $ssl,
                         'ca' => $ca,
-                        'splitread' => Horde_Array::replaceRecursive(
+                        'splitread' => array_replace_recursive(
                             $splitread,
                             array(
                                 'switch' => array(
@@ -1128,7 +1207,7 @@ class Horde_Config
                         'charset' => $charset,
                         'ssl' => $ssl,
                         'ca' => $ca,
-                        'splitread' => Horde_Array::replaceRecursive(
+                        'splitread' => array_replace_recursive(
                             $splitread,
                             array(
                                 'switch' => array(
@@ -1163,7 +1242,10 @@ class Horde_Config
                             '_type' => 'text',
                             'required' => true,
                             'desc' => 'Absolute path to the database file',
-                            'default' => $this->_default($ctx . '|database', '')
+                            'default' => $this->_default(
+                                $ctx . '|database',
+                                $node ? ($xpath->evaluate('string(configstring[@name="database"])', $node) ?: '') : ''
+                            )
                         ),
                         'charset' => $charset
                     )
@@ -1222,6 +1304,7 @@ class Horde_Config
         $default = $node->getAttribute('default');
         $default = empty($default) ? 'horde' : $default;
         list($default, $isDefault) = $this->__default($ctx . '|' . $node->getAttribute('switchname'), $default);
+        $xpath = new DOMXPath($node->ownerDocument);
 
         $config = array(
             'desc' => 'What VFS driver should we use?',
@@ -1235,7 +1318,10 @@ class Horde_Config
                             'vfsroot' => array(
                                 '_type' => 'text',
                                 'desc' => 'Where on the real filesystem should Horde use as root of the virtual filesystem?',
-                                'default' => $this->_default($ctx . '|params|vfsroot', '/tmp')
+                                'default' => $this->_default(
+                                    $ctx . '|params|vfsroot',
+                                    $xpath->evaluate('string(configsection/configstring[@name="vfsroot"])', $node) ?: '/tmp'
+                                )
                             )
                         )
                     )
@@ -1256,30 +1342,44 @@ class Horde_Config
                                 '_type' => 'text',
                                 'required' => true,
                                 'desc' => 'SSH server/host',
-                                'default' => $this->_default($ctx . '|hostspec', '')
+                                'default' => $this->_default(
+                                    $ctx . '|hostspec',
+                                    $xpath->evaluate('string(configsection/configstring[@name="hostspec"])', $node) ?: ''
+                                )
                             ),
                             'port' => array(
                                 '_type' => 'text',
                                 'required' => false,
                                 'desc' => 'Port number on which SSH listens',
-                                'default' => $this->_default($ctx . '|port', '22')
+                                'default' => $this->_default(
+                                    $ctx . '|port',
+                                    $xpath->evaluate('string(configsection/configstring[@name="port"])', $node) ?: '22'
+                                )
                             ),
                             'username' => array(
                                 '_type' => 'text',
                                 'required' => true,
                                 'desc' => 'Username to connect to the SSH server',
-                                'default' => $this->_default($ctx . '|username', '')
+                                'default' => $this->_default(
+                                    $ctx . '|username',
+                                    $xpath->evaluate('string(configsection/configstring[@name="username"])', $node) ?: ''
+                                )
                             ),
                             'password' => array(
                                 '_type' => 'text',
                                 'required' => true,
                                 'desc' => 'Password with which to connect',
-                                'default' => $this->_default($ctx . '|password', '')
+                                'default' => $this->_default(
+                                    $ctx . '|password',
+                                    $xpath->evaluate('string(configsection/configstring[@name="password"])', $node) ?: ''
+                                )
                             ),
                             'vfsroot' => array(
                                 '_type' => 'text',
                                 'desc' => 'Where on the real filesystem should Horde use as root of the virtual filesystem?',
-                                'default' => $this->_default($ctx . '|vfsroot', '/tmp')
+                                'default' => $this->_default(
+                                    $ctx . '|vfsroot',
+                                    $xpath->evaluate('string(configsection/configstring[@name="vfsroot"])', $node) ?: '/tmp')
                             )
                         )
                     )
@@ -1551,13 +1651,21 @@ class Horde_Config
     protected function _handleSpecials($node)
     {
         $app = $node->getAttribute('application');
-        if (!in_array($app, $GLOBALS['registry']->listApps())) {
-            $app = $GLOBALS['registry']->hasInterface($app);
+        try {
+            if (!in_array($app, $GLOBALS['registry']->listApps())) {
+                $app = $GLOBALS['registry']->hasInterface($app);
+            }
+        } catch (Horde_Exception $e) {
+            return array();
         }
         if (!$app) {
             return array();
         }
-        return $GLOBALS['registry']->callAppMethod($app, 'configSpecialValues', array('args' => array($node->getAttribute('name')), 'noperms' => true));
+        try {
+            return $GLOBALS['registry']->callAppMethod($app, 'configSpecialValues', array('args' => array($node->getAttribute('name')), 'noperms' => true));
+        } catch (Horde_Exception $e) {
+            return array();
+        }
     }
 
     /**
