@@ -3,7 +3,7 @@
  * The Horde_Core_Auth_Application class provides application-specific
  * authentication built on top of the horde/Auth API.
  *
- * Copyright 2002-2011 Horde LLC (http://www.horde.org/)
+ * Copyright 2002-2013 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you did
  * not receive this file, see http://opensource.org/licenses/lgpl-2.1.php
@@ -17,15 +17,15 @@
 class Horde_Core_Auth_Application extends Horde_Auth_Base
 {
     /**
-     * Authentication failure reasons (additions to Horde_Auth:: reasons).
-     *
-     * <pre>
-     * REASON_BROWSER - A browser change was detected
-     * REASON_SESSIONIP - Logout due to change of IP address during session
-     * </pre>
+     * Authentication failure reasons (additions to Horde_Auth:: reasons):
+     *   - REASON_BROWSER: A browser change was detected
+     *   - REASON_SESSIONIP: Logout due to change of IP address during session
+     *   - REASON_SESSIONMAXTIME: Logout due to the session exceeding the
+     *                            maximum allowed length.
      */
     const REASON_BROWSER = 100;
     const REASON_SESSIONIP = 101;
+    const REASON_SESSIONMAXTIME = 102;
 
     /**
      * Application for authentication.
@@ -49,12 +49,11 @@ class Horde_Core_Auth_Application extends Horde_Auth_Base
     protected $_base;
 
     /**
-     * The view mode, used to determine if we show dynamic, mobile, traditional
-     * views.
+     * The view mode.
      *
      * @var string
      */
-    protected $_mode = 'auto';
+    protected $_view = 'auto';
 
     /**
      * Available capabilities.
@@ -77,11 +76,9 @@ class Horde_Core_Auth_Application extends Horde_Auth_Base
      * Constructor.
      *
      * @param array $params  Required parameters:
-     * <pre>
-     * 'app' - (string) The application which is providing authentication.
-     * 'base' - (Horde_Auth_Base) The base Horde_Auth driver. Only needed if
-     *          'app' is 'horde'.
-     * </pre>
+     *   - app: (string) The application which is providing authentication.
+     *   - base: (Horde_Auth_Base) The base Horde_Auth driver. Only needed if
+     *           'app' is 'horde'.
      *
      * @throws InvalidArgumentException
      */
@@ -133,9 +130,9 @@ class Horde_Core_Auth_Application extends Horde_Auth_Base
             return false;
         }
 
-        /* Remember the user's mode choice, if applicable */
+        /* Remember the user's mode choice, if applicable. */
         if (!empty($credentials['mode'])) {
-            $this->_mode = $credentials['mode'];
+            $this->_view = $credentials['mode'];
         }
 
         return $this->_setAuth();
@@ -282,7 +279,7 @@ class Horde_Core_Auth_Application extends Horde_Auth_Base
         if ($this->hasCapability('update')) {
             $GLOBALS['registry']->callAppMethod($this->_app, 'authUpdateUser', array('args' => array($oldID, $newID, $credentials)));
         } else {
-            parent::updateUser($userId, $credentials);
+            parent::updateUser($oldID, $newID, $credentials);
         }
     }
 
@@ -437,12 +434,10 @@ class Horde_Core_Auth_Application extends Horde_Auth_Base
      *
      * @param mixed $name  The credential value to get. If null, will return
      *                     the entire credential list. Valid names:
-     * <pre>
-     * 'change' - (boolean) Do credentials need to be changed?
-     * 'credentials' - (array) The credentials needed to authenticate.
-     * 'expire' - (integer) UNIX timestamp of the credential expiration date.
-     * 'userId' - (string) The user ID.
-     * </pre>
+     *   - change: (boolean) Do credentials need to be changed?
+     *   - credentials: (array) The credentials needed to authenticate.
+     *   - expire: (integer) UNIX timestamp of the credential expiration date.
+     *   - userId: (string) The user ID.
      *
      * @return mixed  Return the credential information, or null if the
      *                credential doesn't exist.
@@ -461,12 +456,12 @@ class Horde_Core_Auth_Application extends Horde_Auth_Base
      * @param mixed $value  The credential value to set. See getCredential()
      *                      for the list of valid credentials/types.
      */
-    public function setCredential($type, $value)
+    public function setCredential($name, $value)
     {
         if ($this->_base) {
-            $this->_base->setCredential($type, $value);
+            $this->_base->setCredential($name, $value);
         } else {
-            parent::setCredential($type, $value);
+            parent::setCredential($name, $value);
         }
     }
 
@@ -507,10 +502,8 @@ class Horde_Core_Auth_Application extends Horde_Auth_Base
      *
      * @return array  An array with the following keys:
      * <pre>
-     * 'js_code' - (array) A list of javascript statements to be included via
-     *             Horde::addInlineScript().
-     * 'js_files' - (array) A list of javascript files to be included via
-     *              Horde::addScriptFile().
+     * 'js_code' - (array) A list of javascript statements to be included.
+     * 'js_files' - (array) A list of javascript files to be included.
      * 'params' - (array) A list of parameters to display on the login screen.
      *            Each entry is an array with the following entries:
      *            'label' - (string) The label of the entry.
@@ -576,10 +569,10 @@ class Horde_Core_Auth_Application extends Horde_Auth_Base
 
         try {
             $result = Horde::callHook($type, array($userId, $credentials), $this->_app);
-        } catch (Horde_Exception $e) {
-            throw new Horde_Auth_Exception($e);
         } catch (Horde_Exception_HookNotSet $e) {
             return $ret_array;
+        } catch (Horde_Exception $e) {
+            throw new Horde_Auth_Exception($e);
         }
 
         unset($credentials['authMethod']);
@@ -627,7 +620,7 @@ class Horde_Core_Auth_Application extends Horde_Auth_Base
         /* Destroy any existing session on login and make sure to use a
          * new session ID, to avoid session fixation issues. */
         if (($userId = $registry->getAuth()) === false) {
-            $registry->getCleanSession();
+            $GLOBALS['session']->clean();
             $userId = $this->getCredential('userId');
         }
 
@@ -646,9 +639,10 @@ class Horde_Core_Auth_Application extends Horde_Auth_Base
         ));
 
         /* Only set the view mode on initial authentication */
-        if (!$GLOBALS['session']->get('horde', 'mode')) {
-            $this->_setMode();
+        if (!$GLOBALS['session']->exists('horde', 'view')) {
+            $this->_setView();
         }
+
         if ($this->_base &&
             isset($GLOBALS['notification']) &&
             ($expire = $this->_base->getCredential('expire'))) {
@@ -656,41 +650,94 @@ class Horde_Core_Auth_Application extends Horde_Auth_Base
             $GLOBALS['notification']->push(sprintf(Horde_Core_Translation::ngettext("%d day until your password expires.", "%d days until your password expires.", $toexpire), $toexpire), 'horde.warning');
         }
 
-        $registry->callAppMethod($this->_app, 'authAuthenticateCallback', array('noperms' => true));
-
         return true;
     }
 
     /**
      * Sets the default global view mode in the horde session. This can be
-     * checked by applications, and  overridden if desired. Also sets a cookie
+     * checked by applications, and overridden if desired. Also sets a cookie
      * to remember the last view selection if applicable.
      */
-    protected function _setMode()
+    protected function _setView()
     {
-        global $conf, $browser, $prefs, $registry;
+        global $conf, $browser, $notification, $registry;
+
+        $mode = $this->_view;
 
         if (empty($conf['user']['force_view'])) {
             if (empty($conf['user']['select_view'])) {
-                // No value from login form, try to detect.
-                // THIS IS A HACK. DO PROPER SMARTPHONE DETECTION.
-                if ($browser->isMobile()) {
-                    $this->_mode = $browser->getBrowser() == 'webkit' ? 'smartmobile' : 'mobile';
-                }
+                $mode = 'auto';
             } else {
-                setcookie('default_horde_view', $this->_mode, time() + 30 * 86400, $conf['cookie']['path'], $conf['cookie']['domain']);
-                if ($browser->isMobile() && $this->_mode == 'auto') {
-                    $this->_mode = $browser->getBrowser() == 'webkit' ? 'smartmobile' : 'mobile';
-                }
+                setcookie('default_horde_view', $mode, time() + 30 * 86400, $conf['cookie']['path'], $conf['cookie']['domain']);
             }
         } else {
             // Forcing mode as per config.
-            $this->_mode = $conf['user']['force_view'];
+            $mode = $conf['user']['force_view'];
         }
 
-        // Set it in the session.
-        $GLOBALS['session']->set('horde', 'mode', $this->_mode);
+        /* $mode now contains the user's preference for view based on the
+         * login screen parameters and configuration. */
+        switch ($mode) {
+        case 'auto':
+            if ($browser->hasFeature('ajax')) {
+                $mode = $browser->isMobile()
+                    ? 'smartmobile'
+                    : 'dynamic';
+            } else {
+                $mode = $browser->isMobile()
+                    ? 'mobile'
+                    : 'basic';
+            }
+            break;
+
+        case 'basic':
+            if (!$browser->hasFeature('javascript')) {
+                $notification->push(Horde_Core_Translation::t("Your browser does not support javascript. Using minimal view instead."), 'horde.warning');
+                $mode = 'mobile';
+            }
+            break;
+
+        case 'dynamic':
+            if (!$browser->hasFeature('ajax')) {
+                if ($browser->hasFeature('javascript')) {
+                    $notification->push(Horde_Core_Translation::t("Your browser does not support the dynamic view. Using basic view instead."), 'horde.warning');
+                    $mode = 'basic';
+                } else {
+                    $notification->push(Horde_Core_Translation::t("Your browser does not support the dynamic view. Using minimal view instead."), 'horde.warning');
+                    $mode = 'mobile';
+                }
+            }
+            break;
+
+        case 'smartmobile':
+            if (!$browser->hasFeature('ajax')) {
+                $notification->push(Horde_Core_Translation::t("Your browser does not support the dynamic view. Using minimal view instead."), 'horde.warning');
+                $mode = 'mobile';
+            }
+            break;
+
+        case 'mobile':
+        default:
+            $mode = 'mobile';
+            break;
+        }
+
+        if (($browser->getBrowser() == 'msie') &&
+            ($browser->getMajor() < 8) &&
+            ($mode != 'mobile')) {
+            $notification->push(Horde_Core_Translation::t("You are using an old, unsupported version of Internet Explorer. You must use the minimal view until you upgrade your browser."));
+            $mode = 'mobile';
+        }
+
+        $registry_map = array(
+            'basic' => Horde_Registry::VIEW_BASIC,
+            'dynamic' => Horde_Registry::VIEW_DYNAMIC,
+            'mobile' => Horde_Registry::VIEW_MINIMAL,
+            'smartmobile' => Horde_Registry::VIEW_SMARTMOBILE
+        );
+
+        $this->_view = $mode;
+        $registry->setView($registry_map[$mode]);
     }
 
 }
-
