@@ -3,7 +3,7 @@
  * The Horde_Corde_Auth_Imsp class provides basic authentication against an IMSP
  * server.
  *
- * Copyright 2004-2011 Horde LLC (http://www.horde.org/)
+ * Copyright 2004-2013 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you did
  * not receive this file, see http://opensource.org/licenses/lgpl-2.1.php
@@ -36,7 +36,7 @@ class Horde_Core_Auth_Imsp extends Horde_Auth_Base
         $this->_params['username'] = $userID;
         $this->_params['password'] = $credentials['password'];
         $this->_imsp = $GLOBALS['injector']->getInstance('Horde_Core_Factory_Imsp')->create(null, $this->_params);
-        if (!$result = $this->_imsp->authenticate(false)) {
+        if (!$this->_imsp->authenticate(false)) {
             throw new Horde_Auth_Exception('', Horde_Auth::REASON_BADLOGIN);
         }
     }
