@@ -2,7 +2,7 @@
 /**
  * Factory for creating Horde_Lock objects
  *
- * Copyright 2010 Horde LLC <http://horde.org>
+ * Copyright 2010-2013 Horde LLC (http://www.horde.org/)
  *
  * @category Horde
  * @package  Core
@@ -14,7 +14,7 @@ class Horde_Core_Factory_Lock extends Horde_Core_Factory_Injector
      * Attempts to return a concrete instance based on the configured driver.
      *
      * @return Horde_Lock  The newly created concrete instance.
-     * @throws Horde_Lock_Exception
+     * @throws Horde_Exception
      */
     public function create(Horde_Injector $injector)
     {
@@ -33,13 +33,8 @@ class Horde_Core_Factory_Lock extends Horde_Core_Factory_Injector
             $params['db'] = $injector->getInstance('Horde_Db_Adapter');
         }
 
-        $driver = Horde_String::ucfirst(basename($driver));
-        $class = 'Horde_Lock_' . $driver;
-
-        if (class_exists($class)) {
-            return new $class($params);
-        }
-
-        throw new Horde_Lock_Exception('Horde_Lock driver (' . $class . ') not found');
+        $class = $this->_getDriverName($driver, 'Horde_Lock');
+        return new $class($params);
     }
+
 }
