@@ -2,7 +2,7 @@
 /**
  * The Horde_Core_Mime_Viewer_Vcard class renders out vCards in HTML format.
  *
- * Copyright 2002-2011 Horde LLC (http://www.horde.org/)
+ * Copyright 2002-2013 Horde LLC (http://www.horde.org/)
  *
  * See the enclosed file COPYING for license information (LGPL). If you
  * did not receive this file, see http://www.horde.org/licenses/lgpl21.
@@ -36,15 +36,12 @@ class Horde_Core_Mime_Viewer_Vcard extends Horde_Mime_Viewer_Base
     /**
      * Constructor.
      *
-     * @param Horde_Mime_Part $mime_part  The object with the data to be
-     *                                    rendered.
-     * @param array $conf                 Configuration:
-     * <pre>
-     * 'browser' - (Horde_Browser) Browser object.
-     * 'notification' - (Horde_Notification_Base) Notification object.
-     * 'prefs' - (Horde_Prefs) Prefs object.
-     * 'registry' - (Horde_Registry) Registry object.
-     * </pre>
+     * @param Horde_Mime_Part $part  The object with the data to be rendered.
+     * @param array $conf            Configuration:
+     *   - browser: (Horde_Browser) Browser object.
+     *   - notification: (Horde_Notification_Base) Notification object.
+     *   - prefs: (Horde_Prefs) Prefs object.
+     *   - registry: (Horde_Registry) Registry object.
      *
      * @throws InvalidArgumentException
      */
@@ -70,13 +67,11 @@ class Horde_Core_Mime_Viewer_Vcard extends Horde_Mime_Viewer_Base
         $ret = $this->_renderInline();
 
         if (!empty($ret)) {
-            $templates = $this->getConfigParam('registry')->get('templates', 'horde');
-
             reset($ret);
             Horde::startBuffer();
-            include $templates . '/common-header.inc';
+            $GLOBALS['page_output']->header();
             echo $ret[key($ret)]['data'];
-            include $templates . '/common-footer.inc';
+            $GLOBALS['page_output']->footer();
             $ret[key($ret)]['data'] = Horde::endBuffer();
         }
 
@@ -95,10 +90,8 @@ class Horde_Core_Mime_Viewer_Vcard extends Horde_Mime_Viewer_Base
         $prefs = $this->getConfigParam('prefs');
         $registry = $this->getConfigParam('registry');
 
-        $app = false;
         $data = $this->_mimepart->getContents();
         $html = '';
-        $import_msg = null;
         $title = Horde_Core_Translation::t("vCard");
 
         $iCal = new Horde_Icalendar();
@@ -114,7 +107,7 @@ class Horde_Core_Mime_Viewer_Vcard extends Horde_Mime_Viewer_Base
             foreach ($iCal->getComponents() as $c) {
                 if ($c->getType() == 'vcard') {
                     try {
-                        $contacts = $registry->call('contacts/import', array($c, null, $source));
+                        $registry->call('contacts/import', array($c, null, $source));
                         ++$count;
                     } catch (Horde_Exception $e) {
                         $notification->push(Horde_Core_Translation::t("There was an error importing the contact data:") . ' ' . $e->getMessage(), 'horde.error');
