@@ -1,9 +1,4 @@
 <?php
-/**
- * Require our basic test case definition
- */
-require_once dirname(__FILE__) . '/Autoload.php';
-
 /**
  * @author     Jan Schneider <jan@horde.org>
  * @license    http://www.horde.org/licenses/lgpl21 LGPL 2.1
